import Foundation

// Model & JSON

struct Task {
    let name: String
    let completed: Bool
    let position: Int
    
    static func create(name: String)(completed: Bool)(position: Int) -> Task {
        return Task(name: name, completed: completed, position: position)
    }
}

let json = ["name": "Do this", "completed": false, "position": 10]

// Functional operators

infix operator >>> { associativity left precedence 150 }

func >>> <A, B>(a: A?, f: A -> B?) -> B? {
    if let a = a {
        return f(a)
    } else {
        return nil
    }
}

infix operator <%> { associativity left }

func <%> <A, B>(f: A -> B, a: A?) -> B? {
    if let a = a {
        return f(a)
    } else {
        return nil
    }
}

infix operator <*> { associativity left }

func <*> <A, B>(f: (A -> B)?, a: A?) -> B? {
    if let f = f, a = a {
        return f(a)
    } else {
        return nil
    }
}

// Casting helpers

func autocast<T>(x: AnyObject) -> T? {
    return x as? T
}

// Decoding

let task = Task.create
    <%> json["name"]      >>> autocast
    <*> json["completed"] >>> autocast
    <*> json["position"]  >>> autocast
